@@ -1,6 +1,6 @@
 /* structures.c -- rep's module system
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
-   $Id: structures.c,v 1.28 2001/04/20 18:42:59 jsh Exp $
+   $Id$
 
    This file is part of librep.
 
@@ -147,7 +147,7 @@ DEFSYM(local, "local");
 
 static rep_struct_node *lookup_or_add (rep_struct *s, repv var);
 
-
+
 /* cached lookups */
 
 #ifdef DEBUG
@@ -186,7 +186,7 @@ static struct cache_line ref_cache[CACHE_SETS];
 static inline void
 enter_cache (rep_struct *s, rep_struct_node *binding)
 {
-    u_int hash = CACHE_HASH (binding->symbol);
+    unsigned int hash = CACHE_HASH (binding->symbol);
     if (ref_cache[hash].s != 0)
     {
 #ifdef DEBUG
@@ -203,7 +203,7 @@ enter_cache (rep_struct *s, rep_struct_node *binding)
 static inline rep_struct_node *
 lookup_cache (rep_struct *s, repv var)
 {
-    u_int hash = CACHE_HASH (var);
+    unsigned int hash = CACHE_HASH (var);
     if (ref_cache[hash].s == s && ref_cache[hash].n->symbol == var)
     {
 #ifdef DEBUG
@@ -223,7 +223,7 @@ lookup_cache (rep_struct *s, repv var)
 static inline void
 cache_invalidate_symbol (repv symbol)
 {
-    u_int hash = CACHE_HASH (symbol);
+    unsigned int hash = CACHE_HASH (symbol);
     if (ref_cache[hash].s != 0 && ref_cache[hash].n->symbol == symbol)
 	ref_cache[hash].s = 0;
 }
@@ -268,7 +268,7 @@ static int ref_age;
 static inline void
 enter_cache (rep_struct *s, rep_struct_node *binding)
 {
-    u_int hash = CACHE_HASH (binding->symbol);
+    unsigned int hash = CACHE_HASH (binding->symbol);
     int i, oldest_i, oldest_age = INT_MAX;
     for (i = 0; i < CACHE_ASSOC; i++)
     {
@@ -301,7 +301,7 @@ enter_cache (rep_struct *s, rep_struct_node *binding)
 static inline rep_struct_node *
 lookup_cache (rep_struct *s, repv var)
 {
-    u_int hash = CACHE_HASH (var);
+    unsigned int hash = CACHE_HASH (var);
     int i;
     for (i = 0; i < CACHE_ASSOC; i++)
     {
@@ -323,7 +323,7 @@ lookup_cache (rep_struct *s, repv var)
 static inline void
 cache_invalidate_symbol (repv symbol)
 {
-    u_int hash = CACHE_HASH (symbol);
+    unsigned int hash = CACHE_HASH (symbol);
     int i;
     for (i = 0; i < CACHE_ASSOC; i++)
     {
@@ -391,7 +391,7 @@ cache_flush (void)
 
 #endif /* !SINGLE_DM_CACHE */
 
-
+
 /* type hooks */
 
 static void
@@ -466,7 +466,7 @@ structure_print (repv stream, repv arg)
     }
 }
 
-
+
 /* utilities */
 
 /* Return true iff structure S exports a binding of symbol VAR that it
@@ -636,7 +636,7 @@ rep_search_imports (rep_struct *s, repv var)
     }
 }
 
-
+
 /* lisp functions */
 
 DEFUN("get-structure", Fget_structure,
@@ -1332,34 +1332,51 @@ constant.
     return (n != 0 && n->is_constant) ? Qt : Qnil;
 }
 
-DEFUN ("export-bindings", Fexport_bindings,
-       Sexport_bindings, (repv vars), rep_Subr1)
+repv
+Fexport_binding (repv var)
 {
-    rep_struct *s = rep_STRUCTURE (rep_structure);
-    rep_DECLARE1 (vars, rep_LISTP);
-    while (rep_CONSP (vars) && rep_SYMBOLP (rep_CAR (vars)))
+    rep_struct *s;
+    rep_struct_node *n;
+
+    rep_DECLARE1 (var, rep_SYMBOLP);
+
+    s = rep_STRUCTURE (rep_structure);
+    n = lookup (s, var);
+
+    if (n != 0)
     {
-	repv var = rep_CAR (vars);
-	rep_struct_node *n = lookup (s, var);
-	if (n != 0)
+	if (!n->is_exported)
 	{
-	    if (!n->is_exported)
-	    {
-		n->is_exported = 1;
-		cache_invalidate_symbol (var);
-	    }
-	}
-	else if (!structure_exports_inherited_p (s, var))
-	{
-	    s->inherited = Fcons (var, s->inherited);
+	    n->is_exported = 1;
 	    cache_invalidate_symbol (var);
 	}
-	vars = rep_CDR (vars);
     }
+    else if (!structure_exports_inherited_p (s, var))
+    {
+	s->inherited = Fcons (var, s->inherited);
+	cache_invalidate_symbol (var);
+    }
+
     return Qnil;
 }
 
-
+DEFUN ("export-bindings", Fexport_bindings,
+       Sexport_bindings, (repv vars), rep_Subr1)
+{
+    rep_DECLARE1 (vars, rep_LISTP);
+
+    while (rep_CONSP (vars))
+    {
+	if (Fexport_binding (rep_CAR (vars)) == rep_NULL)
+	    return rep_NULL;
+
+	vars = rep_CDR (vars);
+    }
+
+    return Qnil;
+}
+
+
 /* features */
 
 DEFUN("featurep", Ffeaturep, Sfeaturep, (repv feature), rep_Subr1) /*
@@ -1446,7 +1463,7 @@ loaded is either FILE (if given), or the print name of FEATURE.
     return Qt;
 }
 
-
+
 /* C interface for structure building */
 
 repv
@@ -1636,7 +1653,7 @@ rep_documentation_property (repv structure)
     return Fintern (rep_string_dup (buf), Qnil);
 }
 
-
+
 /* init */
 
 void
