@@ -1,6 +1,6 @@
 /* unix_processes.c -- Subprocess handling for Unix
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: unix_processes.c,v 1.79 2003/10/28 22:29:39 jsh Exp $
+   $Id$
 
    This file is part of Jade.
 
@@ -172,7 +172,7 @@ DEFSTRING(not_local, "Need a local file");
 DEFSTRING(forkstr, "fork");
 DEFSTRING(nosig, "Unknown signal");
 
-
+
 
 static RETSIGTYPE
 sigchld_handler(int sig)
@@ -334,7 +334,7 @@ read_from_one_fd(struct Proc *pr, int fd)
 {
     repv stream = ((fd != pr->pr_Stdout)
 		    ? pr->pr_ErrorStream : pr->pr_OutputStream);
-    u_char buf[1025];
+    char buf[1025];
     int actual;
     do {
 	if((actual = read(fd, buf, 1024)) > 0)
@@ -380,7 +380,7 @@ read_from_process(int fd)
 }
 
 static int
-write_to_process(repv pr, u_char *buf, int bufLen)
+write_to_process(repv pr, char *buf, int bufLen)
 {
     int act = 0;
     if(!PROCESSP(pr))
@@ -472,7 +472,6 @@ static int
 get_pty(char *slavenam)
 {
 #if defined(HAVE_PTYS)
-    char c;
     int master;
 
 # if defined(HAVE_DEV_PTMX) && defined(HAVE_GRANTPT)
@@ -497,7 +496,7 @@ get_pty(char *slavenam)
        The FIRST_PTY_LETTER gives the first X to try. We try in the 
        sequence FIRST_PTY_LETTER, .., 'z', 'a', .., FIRST_PTY_LETTER.
        Is this worthwhile, or just over-zealous? */
-    c = FIRST_PTY_LETTER;
+    char c = FIRST_PTY_LETTER;
     do {
 	int i;
 	for(i = 0; i < 16; i++)
@@ -556,7 +555,7 @@ child_build_environ (void)
    stdin connected to the file SYNC_INPUT. Otherwise this function returns
    immediately after starting the process.  */
 static rep_bool
-run_process(struct Proc *pr, char **argv, u_char *sync_input)
+run_process(struct Proc *pr, char **argv, char *sync_input)
 {
     rep_bool rc = rep_FALSE;
     if(PR_DEAD_P(pr))
@@ -806,7 +805,7 @@ run_process(struct Proc *pr, char **argv, u_char *sync_input)
 		else
 		{
 		    /* Run synchronously.  */
-		    u_char buf[1025];
+		    char buf[1025];
 		    int actual;
 		    fd_set inputs;
 		    rep_bool done_out = rep_FALSE, done_err = rep_FALSE;
@@ -1020,7 +1019,7 @@ static void
 proc_prin(repv strm, repv obj)
 {
     struct Proc *pr = VPROC(obj);
-    u_char buf[40];
+    char buf[40];
     rep_stream_puts(strm, "#<process", -1, rep_FALSE);
     if(PR_RUNNING_P(pr))
     {
@@ -1051,7 +1050,7 @@ static int
 proc_putc(repv stream, int c)
 {
     char tmps[2];
-    tmps[0] = (u_char)c;
+    tmps[0] = (char)c;
     tmps[1] = 0;
     return write_to_process(stream, tmps, 1);
 }
@@ -1059,7 +1058,7 @@ proc_putc(repv stream, int c)
 static int
 proc_puts(repv stream, void *data, int len, rep_bool is_lisp)
 {
-    u_char *buf = is_lisp ? rep_STR(data) : data;
+    char *buf = is_lisp ? rep_STR(data) : data;
     return write_to_process(stream, buf, len);
 }
 
@@ -1117,6 +1116,20 @@ If the DIR parameter is nil it will be inherited from the
 	return rep_mem_error();
 }
 
+DEFUN("close-process", Fclose_process,
+      Sclose_process, (repv proc), rep_Subr1) /*
+::doc:rep.io.processes#close-process::
+close-processes [PROCESS]
+
+Closes the stdin, stdout, and stderr streams of the asynchronous process-
+object PROCESS.
+::end:: */
+{
+    rep_DECLARE1(proc, PROCESSP);
+    close_proc_files(VPROC(proc));
+    return(Qnil); 
+}
+
 DEFUN("start-process", Fstart_process, Sstart_process, (repv arg_list), rep_SubrN) /*
 ::doc:rep.io.processes#start-process::
 start-process [PROCESS] [PROGRAM] [ARGS...]
@@ -2072,6 +2085,7 @@ rep_proc_init(void)
     rep_INTERN(socketpair);
 
     tem = rep_push_structure ("rep.io.processes");
+    rep_ADD_SUBR(Sclose_process);
     rep_ADD_SUBR(Smake_process);
     rep_ADD_SUBR(Sstart_process);
     rep_ADD_SUBR(Scall_process);
